# GPD Library
const1=pi/2
# Prior on Gamma
lpi.gamma=function(gamma,sd=1) sum(dnorm(gamma,mean=0,sd=sd,log=TRUE))


# Link function for xi of a lp linea predictor, must be a cdf in (0,1)
link.cdf=function(lp,z) lp
# Log Transformation
xi.transf=function(lp,link.cdf,z){
	n=length(lp)
	res=rep(-10,n)
	tmp=link.cdf(lp,z)
	res[(tmp>0)&(tmp<1)]=(tan(const1*tmp[(tmp>0)&(tmp<1)])^2-1)/2
	return(res)
}

# Log likelihood function for Regression on Xi and Nu
l.zx=function(y,z,x,beta,gamma,link.cdf,n){
	xi.lp=z%*%gamma # Linear predictor for xi
	xi=xi.transf(xi.lp,link.cdf,z)
    if(any(xi<(-0.5))) return(-Inf)
	nu.lp=x%*%beta # Linear predictor for nu
	nu=exp(nu.lp)
	if(any((1+y*(xi^2+xi)/nu)<=0)) return(-Inf)
	res=rep(-Inf,n)
	exp.case=abs(xi)<1e-10
	nexp.case=!exp.case
	res[exp.case]=-log(nu[exp.case])-y[exp.case]/nu[exp.case]
	res[nexp.case]=log((1+xi[nexp.case]))-log(nu[nexp.case])-(1/xi[nexp.case]+1)*log(1+y[nexp.case]*(xi[nexp.case]^2+xi[nexp.case])/nu[nexp.case])
	return(sum(res))}

# Simulation from a GPD process with x covariates for sigma only
sim.gpd.x=function(n,beta,xi,x){
	nu.lp=x%*%beta # Linear predictor for nu
	nu=exp(nu.lp)
	xi.sigma= xi.nu.to.xi.sigma(cbind(xi,nu))
	y=pareto.simulations(n,xi.sigma[,1],xi.sigma[,2])
	return(y)
}

# Simulation from a GPD process with z,x covariates,
# beta,gamma parameters and a link.cdf
sim.gpd.cov=function(n,beta,gamma,z,x){
	xi.lp=z%*%gamma # Linear predictor for xi
	nu.lp=x%*%beta # Linear predictor for nu
	nu=exp(nu.lp)
	xi=exp(xi.lp)-1
	xi.sigma= xi.nu.to.xi.sigma(cbind(xi,nu))
	y=pareto.simulations(n,xi.sigma[,1],xi.sigma[,2])
	return(y)
}


# Simulation from a GPD process with z,x covariates,
# beta,gamma parameters and a link.cdf
sim.gpd.cov.rand.link=function(n,beta,gamma,z,x,noise){
	xi.lp=z%*%gamma+rnorm(n,sd=noise) # Linear predictor for xi
	nu.lp=x%*%beta+rnorm(n,sd=noise) # Linear predictor for nu
	nu=exp(nu.lp)
	xi=exp(xi.lp)-1
	xi.sigma= xi.nu.to.xi.sigma(cbind(xi,nu))
	y=pareto.simulations(n,xi.sigma[,1],xi.sigma[,2])
	return(y)
}

# Change parametrization from xi.nu to xi.sigma
# return xi.sigma parametrization
xi.nu.to.xi.sigma=function(par) cbind(par[,1],par[,2]/(1+par[,1]))

# Simulations from a Generalized Pareto Distributions
# in xi, sigma parametrization
pareto.simulations<-function(n, xi,  sigma,  u=0){
	res=rep(NA,n)
	exp.case=abs(xi)<1e-10
	nexp.case=!exp.case
    res[exp.case]=rexp(sum(exp.case),1/sigma[exp.case])
    res[nexp.case]=u+(sigma[nexp.case]/xi[nexp.case])*((1-runif(sum(nexp.case)))^(-xi[nexp.case])-1)
    return(res)
}

## CDF of GPD 
##A la funciÛn cdf.gpd hay que pasarle excedencias:
cdf.gpd<-function(x,xi,sigma,u=0){
    x=x-u
    if(x>0){
        if(abs(xi)<1e-10){
            result=1-exp(-x/sigma)
        }else{
            if(xi>0){
                result=1-(1+x*xi/sigma)^(-1/xi)
            }
            else{
                if(x<(-sigma/xi)){ result=1-(1+x*xi/sigma)^(-1/xi)} else{result=1.0}
            }
        
        }   
    }else{
        result=0
    }
    return(result)
}