rm(list=ls(all=TRUE))

# Simulation Function called by mpi.apply in order to run simulations in parallel
sim.n=function(n,noise){
	# Set of Functions for generating data according to a certain design matrix
	source("gpd-library.R")
	library(POT)
	name.file=paste("sj-SinCos-sim-noise",noise,"-n",n,".RData",sep="")
	nint=1000
	cat(name.file)
	nrep=10000
	burn.in=2000
	# True parameters
	beta.true=c(1,0,1,1)
	gamma.true=c(0,0.5,-0.3,0.3)


	true.par=c(beta.true,gamma.true)
	npar=length(true.par)
	mean.post=matrix(NA,nrow=nint,length(true.par))

	if(!any(dir()==name.file)){
	   last.int=1
	   int.length=0
	   coverage=0
	   bias=0
	   mse=0
	}else{load(name.file)}

	# Fixed Design Matrix of SJ Data Model 
	data=read.table(file="sanjuan-daily-m.csv",sep=";",dec=",",header=TRUE)
	# threshold in mm, that can differ for each season:
	u.dry=u.wet=40
	## We clusterize extremes and take the maximum in each cluster 
	tiempo=as.numeric(as.Date(paste(data$Year,data$Month,data$Day,sep="-")))
	tiempo=tiempo-min(tiempo)
	datos=cbind(data$Rainfall,tiempo)
	colnames(datos)=c("obs","time")
	clust.datos=clust(datos,u=30,tim.cond=3,clust.max=TRUE,plot=FALSE)
	data=data[clust.datos[,3],]
	##Season definition
	season=(data$Month%in%c(1:7))
	season[season]="dry" # x=0
	season[season=="FALSE"]="wet" # x=1
	season=factor(season)
	n.orig=dim(data)[1] # Salmple size
	selec=rep(FALSE,n.orig)
	for(i in 1:n.orig) if((season[i]=="dry" && data$Rainfall[i]>u.dry) || (season[i]=="wet" && data$Rainfall[i]>u.wet)) selec[i]=TRUE 
	# Here goes the definition of the final DataSet
	data=data[selec,]
	# Final Season Vector
	season=(data$Month%in%c(1:7))
	season[season]="dry" # x=0
	season[season=="FALSE"]="wet" # x=1
	season=factor(season)
	y=data$Rainfall
	ct=cos(data$Year-min(data$Year))
	st=sin(data$Year-min(data$Year))

	# These are the original design matrix from SanJuan Data
	mylm=lm(y~season+ct+st,x=TRUE)	
	z.orig=mylm$x
	x.orig=z.orig 
	#mylm=lm(y~1,x=TRUE)


	p=ncol(x.orig)
	q=ncol(z.orig)
	n.orig=nrow(x.orig)

	# Set up for MCMC: definition of vectors of results
	totsim=burn.in+nrep
	varlabel=c(paste("beta[",1:p,"]",sep=""),paste("gamma[",1:q,"]",sep=""))
	theta=array(NA,dim=c(totsim,p+q))
	theta.star=rep(NA,p+q)
	colnames(theta)=varlabel
	fv=theta.star=rep(NA,p+q)
	theta.star=lr=sampord=array(NA,dim=c(totsim,p+q))

	# posterior kernel
	l.zx=function(y,z,x,beta,gamma,n){
		xi=exp(z%*%gamma)-1
	    if(any(xi<(-0.5))) return(-Inf)
		nu=exp(x%*%beta)
		if(any((1+y*(xi^2+xi)/nu)<=0)) return(-Inf)
		return(sum(log((1+xi))-log(nu)-(1/xi+1)*log(1+y*(xi^2+xi)/nu)))
		}

	kp=function(theta) l.zx(y,z,x,beta=theta[1:p],gamma=theta[(p+1):(p+q)],n)

	# Cycle for Intervals
	for(last in last.int:nint){
	######################## SIM CORE ###########################
	# We resample with repetition from the design matrices
	sampled=sort(sample(n.orig,size=n,replace=TRUE))
	x=as.matrix(x.orig[sampled,],ncol=p,nrow=n)
	z=as.matrix(z.orig[sampled,],ncol=q,nrow=n)
	
	
	# Simulation of y: one for each line of the design matrix
	y=Inf
	while(any(y==Inf)) y=sim.gpd.cov.rand.link(n,beta.true,gamma.true,z,x,noise)


	# Jumping distribution (variance)
	if(n==50) sigma.jump=c(1,1,1,1,1,1,1,1)/2
	if(n==100) sigma.jump=c(1,1,1,1,1,1,1,1)/3
	if(n==200) sigma.jump=c(1,1,1,1,1,1,1,1)/5
	acc=rep(0,p+q)
	#theta[1,]= true.par
	theta[1,]=rep(0.1,p+q)
	
	# Generation of Candidates
	for(i in 2:totsim){
		lr[i,]=log(runif(p+q))
		sampord[i,]=sample(p+q,size=p+q)
		theta.star[i,]=rnorm(p+q,mean=0,sd=sigma.jump)
	}
	# The MCMC
	for(i in 2:totsim){
		theta.star[i,]=theta.star[i,]+theta[i-1,]
		theta[i,]=theta[i-1,]
		kpold=kp(theta[i-1,])
		for(j in sampord[i,]){fv[-j]=theta[i,-j];fv[j]=theta.star[i,j];if(lr[i,j]<(kp(fv)-kpold)) theta[i,j]=theta.star[i,j]}
		acc=acc+(theta[i,]!=theta[i-1,])
		if(i==(totsim)) cat("i=",i,"AR=",round(acc/i,1)*10,"\n")
	}
	
#  par(mfrow=c(4,3))
#	for(j in 1:p){
#		 plot.ts(theta[,j],main=varlabel[j])
#	 	 acf(theta[,j],main=varlabel[j])
#		 hist(theta[,j],main=varlabel[j])
#	}
#	quartz()
#	par(mfrow=c(4,3))
#	for(j in (p+1):(p+q)){
#		 plot.ts(theta[,j],main=varlabel[j])
#	 	 acf(theta[,j],main=varlabel[j])
#		 hist(theta[,j],main=varlabel[j])
#	}

	all.mpost=theta[-(1:burn.in),]
	########################################################################

	########################################################################
	# Interval Analysis
	########################################################################

	# CI Analysis
	credible.int=apply(all.mpost,2,function(x) quantile(x,p=c(0.025,0.975)))
	int.length.sim=credible.int[2,]-credible.int[1,]
	inside.int=as.numeric((credible.int[1,]<true.par)&(credible.int[2,]>true.par))

	# Bias and MSE Analysis
	mean.post[last,]=apply(all.mpost,2,mean)
	bias.sim=true.par-mean.post[last,]
	#The MSE is calculated at the end
	#mse.sim=sqrt(apply((all.mpost-true.par)^2,2,mean))
	

	if(last>1){
	 int.length=(int.length*(last-1)+int.length.sim)/last
	 coverage=(coverage*(last-1)+inside.int)/last
	 bias=(bias*(last-1)+bias.sim)/last
	 #mse=(mse*(last-1)+mse.sim)/last
	}else{
	 int.length=int.length.sim
	 coverage=inside.int
	 bias=bias.sim
	 #mse=mse.sim
	}
	cat("last=",last,"/",nint,"\n")
	last.int=last
	save(last.int,int.length,coverage,bias,mean.post,file=name.file)
	}
}

ns=c(50,100,200)
library(Rmpi)
mpi.spawn.Rslaves(nslaves=length(ns))
mpi.apply(ns,sim.n,noise=0)
mpi.apply(ns,sim.n,noise=0.1)
mpi.apply(ns,sim.n,noise=0.5)
mpi.close.Rslaves()



## Coverage Analysis and Graphs
p=4
q=4
##with sin/cos 
beta.true=c(1,0,1,1)
gamma.true=c(0,0.5,-0.3,0.3)
ns=c(50,100,200)


true.par=c(beta.true,gamma.true)
nint=1000

varlabel=c(paste("beta[",1:p,"]",sep=""),paste("gamma[",1:q,"]",sep=""))
int.n=bias.n=mse.n=cov.n=array(NA,dim=c(length(ns),p+q),dimnames=list(ns,varlabel))

pdf("sj-SinCos-sim.pdf")
for(noise in c(0,0.1,0.5)){
#x11()
	for(n in ns){
		ind=which(ns==n)
		name.file=paste("sj-SinCos-sim-noise",noise,"-n",n,".RData",sep="")
  		load(name.file)
  		cov.n[ind,]=round(coverage,3)[1:(p+q)]
  		int.n[ind,]=round(int.length,3)[1:(p+q)]
  		bias.n[ind,]=round(bias,3)[1:(p+q)]
  		mse.n[ind,]=0
  		for(i in 1:nint) mse.n[ind,]=mse.n[ind,]+(mean.post[i,]-true.par)^2
  		mse.n[ind,]=mse.n[ind,]/nint
  	}
	sd.cov=sqrt(0.95*0.05/500)
	ci.lim=0.95+3*c(1,-1)*sd.cov

#	postscript(file=paste("cov-analysis-sanjuan",noise,".ps",sep=""),horizontal=TRUE)
	par(mfrow=c(2,2))

	matplot(ns,cov.n,type="b",main="(a) Coverage",col=1,lwd=2,lty=1:8,pch=c(19,21,19,21,22,23,24,25),ylab="Coverage",xlab="n", axes=TRUE)
#	axis(1,ns)
#	axis(2,seq(0.9,1,by=0.025))
#abline(h=ci.lim,lty=3,lwd=1)

	matplot(ns,int.n,type="b",main="(b) Interval Length",col=1,lwd=2,lty=1:8,pch=c(19,21,19,21,22,23,24,25),ylab="Interval Mean Length",xlab="n",axes=TRUE,ylim=c(0.5,2.5))
#	axis(1,ns)
#	axis(2,seq(1,12,by=1))
	legend(120,2,expression(beta[1],beta[2],beta[3],beta[4],gamma[1],gamma[2],gamma[3],gamma[4]),bty="n",lwd=2,lty=1:8,pch=c(19,21,19,21,22,23,24,25),ncol=2)


	matplot(ns,bias.n,type="b",main="(c) Posterior Mean BIAS",col=1,lwd=2,lty=1:8,pch=c(19,21,19,21,22,23,24,25),ylab="Bias",xlab="n",axes=TRUE)
#axis(1,ns)
#axis(2,seq(-0.5,0.5,by=.1))

	matplot(ns,abs(mse.n),type="b",main="(d) Posterior Mean MSE",col=1,lwd=2,lty=1:8,pch=c(19,21,19,21,22,23,24,25),axes=TRUE,ylab="MSE",xlab="n")
#axis(1,ns)
#axis(2,seq(0,4,by=1))
} # End noise loop

dev.off()

