rm(list=ls(all=TRUE))


# Simulation Function called by mpi.apply in order to run simulations in parallel
sim.n.intensity=function(nyears,noise){
	source("../gpd-library.R")
	library(POT)
	name.file=paste("sj-intensity-sim-noise",noise,"-nyears",nyears,".RData",sep="")
	nint=1000
	cat(name.file)
	nrep=10000
	burn.in=2000
	# True parameters
	alpha.true=c(1.55,0.7,-0.2,1.2)
	
	true.par=c(alpha.true)
	npar=length(true.par)
	mean.post=matrix(NA,nrow=nint,length(true.par))

	if(!any(dir()==name.file)){
	   last.int=1
	   int.length=0
	   coverage=0
	   bias=0
	   mse=0
	}else{load(name.file)}


	#Design matrix for a similar case to San Juan data in which we simulate also dependence on time.
	##Intensity depends on time through a wave cos/sin
	ti=as.numeric(gl(nyears,2))
	cti=cos(ti)
	sti=sin(ti)
	myseason=as.factor(rep(c("dry","wet"),nyears)) # 0=dry, 1=wet
	tmp=runif(2*nyears)

	#Design matrix for lambda:
	ws=lm(tmp~myseason+cti+sti,x=TRUE)$x
	deltai=rep(c(7,5),nyears)/12
	
	#ws=lm(tmp~myseason,x=TRUE)$x
	#deltai=rep(c(7,5),nyears)/12

	##We are considering nyears years (from 1967 to 1967+nyears) with two seasons, each season with a different length that is reflected in deltai
	
		
	#We consider that the intensity lambda changes with time and also with season:
	r=4
	
	
	# Set up for MCMC: definition of vectors of results
	totsim=burn.in+nrep
	varlabel=c(paste("alpha[",1:r,"]",sep=""))
	theta=array(NA,dim=c(totsim,r))
	theta.star=rep(NA,r)
	colnames(theta)=varlabel
	sampord=theta.star=rep(NA,r)
	
	# Poisson Process intensity
	l.poisson=function(alpha,counts,deltai=deltai,ws) sum(counts*log((ws%*%alpha)^2))-sum(deltai*(ws%*%alpha)^2)


	
	
	# Cycle for Intervals
	for(last in last.int:nint){
	######################## SIM CORE ###########################
	
	
	# Simulation of counts
	nlambdas=dim(ws)[1]
	lp.lambdas=ws%*%alpha.true+rnorm(nlambdas,sd=noise)
	lambdas=(lp.lambdas)^2
	counts=rep(NA,length(lambdas))
	for(i in 1:length(lambdas)) counts[i]=rpois(1,lambda=deltai[i]*lambdas[i])

	# Jumping distribution (variance)

	if(nyears==35) 	sigma.jump=c(1,1,1,1)/2
	
	if(nyears==50) 	sigma.jump=c(1,1,1,1)/3

	if(nyears==100) 	sigma.jump=c(1,1,1,1)/4
		
	if(nyears==150) 	sigma.jump=c(1,1,1,1)/5
	
	#if(nyears==200) 	sigma.jump=c(1,1,1,1)/6
	
	#if(nyears==300) 	sigma.jump=c(1,1,1,1)/7
	
	acc=rep(0,r)
	theta[1,]= true.par+0.1
	
	# Generation of Candidates
	
	for(i in 2:totsim){
		sampord=sample(r,size=r)
		theta.star=theta[i-1,]
	for(j in sampord){
		theta.star[j]=rnorm(1,mean=theta[i-1,j],sd=sigma.jump[j])
	    lratio=l.poisson(alpha=theta.star,counts,deltai=deltai,ws=ws)-l.poisson(alpha=theta[i-1,],counts,deltai=deltai,ws=ws)
	    if(log(runif(1))>lratio) theta.star[j]= theta[i-1,j]
		}
		theta[i,]=theta.star
		acc=acc+(theta[i,]!=theta[i-1,])
		if(i==totsim) cat("i=",i,"AR=",round(acc/i,1)*10,"\n")
	}

	all.mpost=theta[-(1:burn.in),]
	
	
#	par(mfrow=c(4,3))
#	for(j in 1:r){
#		 plot.ts(theta[,j],main=varlabel[j])
#	 	 acf(theta[,j],main=varlabel[j])
#		 hist(theta[,j],main=varlabel[j])
#	}


#
#lp.lambda=ws%*%t(theta)
#lambda=lp.lambda^2
#
#par(mfrow=c(1,2))
#hist(lambda[ws[,2]==0,])
#abline(v=lambdas[1])
#
#hist(lambda[ws[,2]==1,])
#abline(v=lambdas[2])
#
#
#(sum(counts[ws[,2]==0])+0.5)/sum(deltai[ws[,2]==0])
#
#(sum(counts[ws[,2]==1])+0.5)/sum(deltai[ws[,2]==1])
#
#
	
	########################################################################

	########################################################################
	# Interval Analysis
	########################################################################

	# CI Analysis
	credible.int=apply(all.mpost,2,function(x) quantile(x,p=c(0.025,0.975)))
	int.length.sim=credible.int[2,]-credible.int[1,]
	inside.int=as.numeric((credible.int[1,]<true.par)&(credible.int[2,]>true.par))

	# Bias and MSE Analysis
	mean.post[last,]=apply(all.mpost,2,mean)
	bias.sim=true.par-mean.post[last,]
	#The MSE is calculated at the end
	#mse.sim=sqrt(apply((all.mpost-true.par)^2,2,mean))
	

	if(last>1){
	 int.length=(int.length*(last-1)+int.length.sim)/last
	 coverage=(coverage*(last-1)+inside.int)/last
	 bias=(bias*(last-1)+bias.sim)/last
	 #mse=(mse*(last-1)+mse.sim)/last
	}else{
	 int.length=int.length.sim
	 coverage=inside.int
	 bias=bias.sim
	 #mse=mse.sim
	}
	cat("last=",last,"/",nint,"\n")
	last.int=last
	save(last.int,int.length,coverage,bias,mean.post,file=name.file)
	}
}



nyears=c(35,50,100,150)
library(Rmpi)
mpi.spawn.Rslaves(nslaves=length(nyears))
#mpi.apply(nyears,sim.n.intensity,noise=0)
mpi.apply(nyears,sim.n.intensity,noise=0.1)
#mpi.apply(nyears,sim.n.intensity,noise=1)
mpi.close.Rslaves()




## Coverage Analysis and Graphs
r=4
##with sin/cos 
alpha.true=c(1.55,0.7,-0.2,1.2)

true.par=c(alpha.true)
nint=1000
nyears=c(35,50,100,150)
varlabel=c(paste("alpha[",1:r,"]",sep=""))
int.n=bias.n=mse.n=cov.n=array(NA,dim=c(length(nyears),r),dimnames=list(nyears,varlabel))

pdf("sj-intensity-sin-cos-sim.pdf")
for(noise in c(0,0.1,0.5)){
	for(n in nyears){
		ind=which(nyears==n)
		name.file=paste("sj-intensity-sim-noise",noise,"-nyears",n,".RData",sep="")
		load(name.file)
  		cov.n[ind,]=round(coverage,3)[1:r]
  		int.n[ind,]=round(int.length,3)[1:r]
  		bias.n[ind,]=round(bias,3)[1:r]
  		mse.n[ind,]=0
  		for(i in 1:nint) mse.n[ind,]=mse.n[ind,]+(mean.post[i,]-true.par)^2
  		mse.n[ind,]=mse.n[ind,]/nint
  	}
	sd.cov=sqrt(0.95*0.05/500)
	ci.lim=0.95+3*c(1,-1)*sd.cov

#	postscript(file=paste("cov-analysis-sanjuan",noise,".ps",sep=""),horizontal=TRUE)
#quartz()
	par(mfrow=c(2,2))

	matplot(nyears,cov.n,type="b",main="(a) Coverage",col=1,lwd=2,lty=1:4,pch=c(19,21,19,22),ylab="Coverage",xlab="n", axes=TRUE)
#	axis(1,ns)
#	axis(2,seq(0.9,1,by=0.025))
#abline(h=ci.lim,lty=3,lwd=1)

	matplot(nyears,int.n,type="b",main="(b) Interval Length",col=1,lwd=2,lty=1:4,pch=c(19,21,19,22),ylab="Interval Mean Length",xlab="n",axes=TRUE)
#	axis(1,ns)
#	axis(2,seq(1,12,by=1))
	legend(50,3.5,colnames(cov.n),bty="n",ncol=1,lwd=2,lty=1:4,pch=c(19,21,19,22))


	matplot(nyears,bias.n,type="b",main="(c) Posterior Mean BIAS",col=1,lwd=2,lty=1:4,pch=c(19,21,19,22),ylab="Bias",xlab="n",axes=TRUE)
#axis(1,ns)
#axis(2,seq(-0.5,0.5,by=.1))

	matplot(nyears,abs(mse.n),type="b",main="(d) Posterior Mean MSE",col=1,lwd=2,lty=1:4,pch=c(19,21,19,22),axes=TRUE,ylab="MSE",xlab="n",ylim=c(0,0.05))
#axis(1,ns)
#axis(2,seq(0,4,by=1))
legend(100,0.05,expression(alpha[1],alpha[2],alpha[3],alpha[4]),lty=1:4,pch=c(19,21,19,22),bty="n")
} # End noise loop

dev.off()





